@@ -2,39 +2,32 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 7)
+__version__ = (0, 0, 8)
 __all__ = [
-    "create_cookie", "create_morsel", "cookie_to_morsel", "morsel_to_cookie", 
-    "extract_cookies", "update_cookies", "cookies_str_to_dict", "cookies_dict_to_str", 
+    "create_cookie", "create_morsel", "to_cookie", "to_morsel", 
+    "cookie_to_morsel", "morsel_to_cookie", "cookies_str_to_dict", 
+    "cookies_dict_to_str", "extract_cookies", "update_cookies", 
     "iter_resp_cookies", 
 ]
 
 from calendar import timegm
-from collections.abc import (
-    Buffer, ItemsView, Iterable, Iterator, Mapping, Sequence, ValuesView, 
-)
+from collections.abc import Buffer, Iterable, Iterator, Mapping, Sequence
 from copy import copy
 from datetime import datetime
-from http.client import HTTPMessage
+from functools import partial
 from http.cookiejar import Cookie, CookieJar
 from http.cookies import Morsel, SimpleCookie
 from re import compile as re_compile
 from time import gmtime, strftime, strptime, time
-from typing import cast, Any, Protocol
+from typing import cast, Any
 from urllib.request import Request
 
-from dicttools import iter_items
+from dicttools import get, iter_items
 
 
 CRE_COOKIE_SEP_split = re_compile(r";\s*").split
 
 
-class HasHeaders[AnyStr: (bytes, str)](Protocol):
-    @property
-    def headers(self, /) -> HTTPMessage | Mapping[AnyStr, AnyStr] | Iterable[tuple[AnyStr, AnyStr]]:
-        ...
-
-
 def create_cookie(
     name: str, 
     value: str | Cookie | Morsel | Mapping, 
@@ -50,19 +43,15 @@ def create_cookie(
             return cookie
         kwargs = {**value.__dict__, **kwargs}
         kwargs.setdefault("rest", kwargs.pop("_rest", {"HttpOnly": None}))
-        value = kwargs["value"]
+        value = value.value or ""
     elif isinstance(value, Morsel):
         kwargs = {
-            "comment": value["comment"], 
-            "discard": False, 
-            "domain": value["domain"], 
-            "path": value["path"], 
+            **value, 
             "rest": {
-                "HttpOnly": value["httponly"], 
-                "SameSite": value["samesite"], 
+                "HttpOnly": value.get("httponly"), 
+                "SameSite": value.get("samesite"), 
+                "Max-Age": value.get("max-age"), 
             }, 
-            "secure": bool(value["secure"]), 
-            "version": value["version"] or 0, 
             **kwargs, 
         }
         value = value.value
@@ -70,25 +59,19 @@ def create_cookie(
         kwargs = {**value, **kwargs}
         value = kwargs.get("value", "")
     if not name:
-        name = kwargs.get("name", "")
-    expires = kwargs.get("expires")
-    max_age = kwargs.get("max-age")
-    if expires:
-        if isinstance(expires, int):
-            pass
-        elif isinstance(expires, datetime):
-            expires = expires.timestamp()
-        else:
-            if len(expires) == 27:
-                fmt = "%a, %d-%b-%y %H:%M:%S GMT"
-            else:
-                fmt = "%a, %d-%b-%Y %H:%M:%S GMT"
-            expires = timegm(strptime(expires, fmt))
-    elif max_age:
+        name = cast(str, kwargs["name"])
+    if not name:
+        raise ValueError(f"please provide a name for value {value!r}")
+    if expires := kwargs.get("expires"):
+        if isinstance(expires, datetime):
+            kwargs["expires"] = expires.timestamp()
+        elif isinstance(expires, str):
+            kwargs["expires"] = timegm(strptime(expires, "%a, %d-%b-%Y %H:%M:%S GMT"))
+    elif max_age := kwargs.get("max-age"):
         try:
-            expires = int(time()) + int(max_age)
+            kwargs["expires"] = int(time()) + int(max_age)
         except ValueError:
-            raise TypeError(f"max-age: {max_age} must be integer")
+            raise TypeError(f"max-age: {max_age!r} must be integer")
     result: dict[str, Any] = {
         "version": 0, 
         "name": name, 
@@ -104,12 +87,7 @@ def create_cookie(
         "rest": {"HttpOnly": None}, 
         "rfc2109": False, 
     }
-    result.update(
-        (key, val)
-        for key, val in kwargs.items()
-        if key in result
-    )
-    result["comment_url"] = bool(result["comment"])
+    result.update(e for e in kwargs.items() if e[0] in result)
     result["port_specified"] = bool(result["port"])
     result["domain_specified"] = bool(result["domain"])
     result["domain_initial_dot"] = result["domain"].startswith(".")
@@ -130,8 +108,10 @@ def create_morsel(
         kwargs = {**value.__dict__, **kwargs}
         kwargs.setdefault("rest", kwargs.pop("_rest", {"HttpOnly": None}))
         if not name:
-            name = kwargs.get("name", "")
-        value = cast(str, kwargs["value"])
+            name = cast(str, kwargs["name"])
+        if not name:
+            raise ValueError(f"please provide a name for value {value!r}")
+        value = cast(str, kwargs.get("value") or "")
         morsel = Morsel()
         morsel.set(name, value, value)
     elif isinstance(value, Morsel):
@@ -143,46 +123,134 @@ def create_morsel(
     else:
         kwargs = {**value, **kwargs}
         if not name:
-            name = kwargs.get("name", "")
-        value = cast(str, kwargs.get("value", ""))
+            name = cast(str, kwargs["name"])
+        if not name:
+            raise ValueError(f"please provide a name for value {value!r}")
+        value  = cast(str, kwargs.get("value") or "")
         morsel = Morsel()
         morsel.set(name, value, value)
-    expires = kwargs.get("expires")
-    if expires:
+    if expires := kwargs.get("expires"):
         if isinstance(expires, datetime):
             morsel["expires"] = expires.strftime("%a, %d-%b-%Y %H:%M:%S GMT")
-        else:
+        elif isinstance(expires, (int, float)):
             morsel["expires"] = strftime("%a, %d-%b-%Y %H:%M:%S GMT", gmtime(expires))
-    if "path" in kwargs or morsel["path"] in ("", None):
-        morsel["path"] = kwargs.get("path", "/")
-    if "comment" in kwargs:
-        morsel["comment"] = kwargs["comment"]
-    if "domain" in kwargs:
-        morsel["domain"] = kwargs["domain"]
-    if "secure" in kwargs or morsel["secure"] in ("", None):
-        morsel["secure"] = kwargs.get("secure", False)
-    if "version" in kwargs or morsel["version"] in ("", None):
-        morsel["version"] = kwargs.get("version", 0)
-    rest = kwargs.get("rest")
-    if rest:
-        if "HttpOnly" in rest:
-            morsel["httponly"] = rest["HttpOnly"]
-        if "Max-Age" in rest:
-            morsel["max-age"]  = rest["Max-Age"]
-        if "SameSite" in rest:
-            morsel["samesite"] = rest["SameSite"]
+    setdefault = morsel.setdefault
+    setdefault("expires", kwargs.get("expires") or "")
+    setdefault("path", kwargs.get("path") or "")
+    setdefault("comment", kwargs.get("comment") or "")
+    setdefault("domain", kwargs.get("domain") or "")
+    setdefault("secure", kwargs.get("secure"))
+    setdefault("version", kwargs.get("version"))
+    rest = kwargs.pop("rest", None) or {}
+    setdefault("httponly", rest.get("HttpOnly"))
+    setdefault("max-age", rest.get("Max-Age", ""))
+    setdefault("samesite", rest.get("SameSite", ""))
     return morsel
 
 
+def to_cookie(cookie, /, name: str = "") -> Cookie:
+    if isinstance(cookie, str):
+        if not name:
+            raise ValueError(f"please provide a name for value {cookie!r}")
+        return create_cookie(name, cookie)
+    elif isinstance(cookie, Morsel):
+        return morsel_to_cookie(cookie)
+    elif isinstance(cookie, Cookie):
+        cookie = copy(cookie)
+        if name:
+            cookie.name = name
+        return cookie
+    else:
+        if isinstance(cookie, Mapping):
+            getval = partial(get, cookie)
+        else:
+            getval = partial(getattr, cookie)
+        name = getval("name", name)
+        if not name:
+            raise ValueError(f"please provide a name for value {cookie!r}")
+        kwargs = {
+            "name": name, 
+            "value": getval("value", ""), 
+            "version": getval("version", None) or getval("Version", None), 
+            "expires": getval("expires", None), 
+            "port": getval("port", None), 
+            "domain": getval("domain", "") or getval("Domain", ""), 
+            "domain_initial_dot": getval("domain_initial_dot", ""), 
+            "path": getval("path", ""), 
+            "secure": getval("secure", "") or getval("Secure", ""), 
+            "discard": getval("discard", ""), 
+            "comment": getval("comment", "") or getval("Comment", ""), 
+            "comment_url": getval("comment_url", ""), 
+            "rest": getval("rest", None) or {}, 
+        }
+        rest: dict = kwargs["rest"]
+        for keys in [
+            ("Max-Age", "max-age", "max_age"), 
+            ("HttpOnly", "httponly", "http_only"), 
+            ("SameSite", "samesite", "same_site"), 
+        ]:
+            for key in keys:
+                val = getval(key, None)
+                if val is not None:
+                    rest[keys[0]] = str(val)
+                    break
+    return create_cookie(**kwargs)
+
+
+def to_morsel(cookie, /, name: str = "") -> Morsel:
+    if isinstance(cookie, str):
+        if not name:
+            raise ValueError(f"please provide a name for value {cookie!r}")
+        return create_morsel(name, cookie)
+    elif isinstance(cookie, Morsel):
+        morsel = copy(cookie)
+        if name:
+            setattr(morsel, "_key", name)
+        return morsel
+    elif isinstance(cookie, Cookie):
+        return cookie_to_morsel(cookie)
+    else:
+        if isinstance(cookie, Mapping):
+            getval = partial(get, cookie)
+        else:
+            getval = partial(getattr, cookie)
+        name = getval("name", name)
+        if not name:
+            raise ValueError(f"please provide a name for value {cookie!r}")
+        kwargs = {
+            "name": name, 
+            "value": getval("value", ""), 
+            "version": getval("version", None) or getval("Version", None), 
+            "expires": getval("expires", None), 
+            "domain": getval("domain", "") or getval("Domain", ""), 
+            "path": getval("path", ""), 
+            "secure": getval("secure", "") or getval("Secure", ""), 
+            "comment": getval("comment", "") or getval("Comment", ""), 
+            "rest": getval("rest", None) or {}, 
+        }
+        rest: dict = kwargs["rest"]
+        for keys in [
+            ("Max-Age", "max-age", "max_age"), 
+            ("HttpOnly", "httponly", "http_only"), 
+            ("SameSite", "samesite", "same_site"), 
+        ]:
+            for key in keys:
+                val = getval(key, None)
+                if val is not None:
+                    rest[keys[0]] = str(val)
+                    break
+        return create_morsel(**kwargs)
+
+
 def cookie_to_morsel(cookie: Cookie, /) -> Morsel:
     morsel: Morsel = Morsel()
     value = cookie.value or ""
     morsel.set(cookie.name, value, value)
-    expires: Any = cookie.expires
-    if expires:
+    expires: Any
+    if expires := cookie.expires:
         if isinstance(expires, datetime):
             expires = expires.strftime("%a, %d-%b-%Y %H:%M:%S GMT")
-        else:
+        elif isinstance(expires, (int, float)):
             expires = strftime("%a, %d-%b-%Y %H:%M:%S GMT", gmtime(expires))
     rest = getattr(cookie, "_rest", {})
     morsel.update({
@@ -200,15 +268,12 @@ def cookie_to_morsel(cookie: Cookie, /) -> Morsel:
 
 
 def morsel_to_cookie(cookie: Morsel, /) -> Cookie:
-    expires = cookie["expires"] or None
-    max_age = cookie["max-age"] or None
-    if expires:
-        if len(expires) == 27:
-            fmt = "%a, %d-%b-%y %H:%M:%S GMT"
-        else:
-            fmt = "%a, %d-%b-%Y %H:%M:%S GMT"
-        expires = timegm(strptime(expires, fmt))
-    elif max_age:
+    if expires := cookie.get("expires"):
+        if isinstance(expires, datetime):
+            expires = expires.timestamp()
+        elif isinstance(expires, str):
+            expires = timegm(strptime(expires, "%a, %d-%b-%Y %H:%M:%S GMT"))
+    elif max_age := cookie.get("max-age"):
         try:
             expires = int(time()) + int(max_age)
         except ValueError:
@@ -231,25 +296,46 @@ def morsel_to_cookie(cookie: Morsel, /) -> Cookie:
     )
 
 
-def extract_cookies[Cookies: (CookieJar, SimpleCookie)](
-    cookies: Cookies, 
+def cookies_str_to_dict(cookies: str, /) -> dict[str, str]:
+    return dict(cookie.split("=", 1) for cookie in CRE_COOKIE_SEP_split(cookies) if cookie)
+
+
+def cookies_dict_to_str(
+    cookies: CookieJar | SimpleCookie | Mapping[str, Any] | Iterable[Any], 
+    /, 
+) -> str:
+    if isinstance(cookies, CookieJar):
+        cookie_it: Iterator[tuple[str, None | str]] = ((cookie.name, cookie.value) for cookie in cookies)
+    elif isinstance(cookies, SimpleCookie):
+        cookie_it = ((name, morsel.value) for name, morsel in cookies.items())
+    elif isinstance(cookies, Mapping):
+        cookie_it = ((name, cookie if isinstance(cookie, str) else cookie.value) for name, cookie in iter_items(cookies))
+    else:
+        cookie_it = (
+            cookie[:2] if isinstance(cookie, tuple) else 
+                ((cookie.key if isinstance(cookie, Morsel) else getattr(cookie, "name", "")), getattr(cookie, "value", None)) 
+            for cookie in cookies
+        )
+    return "; ".join(f"{key}={val}" for key, val in cookie_it if key and val is not None)
+
+
+def extract_cookies[T: (SimpleCookie, CookieJar)](
+    cookies: T, 
     url: str, 
-    response: HasHeaders, 
-) -> Cookies:
-    if not hasattr(response, "info"):
+    response, 
+) -> T:
+    if hasattr(response, "headers"):
         headers = response.headers
-        if not isinstance(headers, HTTPMessage):
-            headers_old = headers
-            headers = HTTPMessage()
-            if isinstance(headers_old, Mapping):
-                headers_old = ItemsView(headers_old)
-            for k, v in headers_old:
-                if isinstance(k, bytes):
-                    k = str(k, "latin-1")
-                if isinstance(v, bytes):
-                    v = str(v, "latin-1")
-                headers.add_header(k, v)
-        setattr(response, "info", lambda: headers)
+    elif hasattr(response, "getheaders"):
+        headers = response.getheaders()
+    elif hasattr(response, "cookies"):
+        update_cookies(cookies, response.cookies)
+        return cookies
+    elif isinstance(response, (CookieJar, SimpleCookie)):
+        update_cookies(cookies, response)
+        return cookies
+    else:
+        headers = response
     if isinstance(cookies, SimpleCookie):
         cookiejar = CookieJar()
         cookiejar.extract_cookies(response, Request(url)) # type: ignore
@@ -259,55 +345,38 @@ def extract_cookies[Cookies: (CookieJar, SimpleCookie)](
     return cookies
 
 
-def update_cookies[Cookies: (CookieJar, SimpleCookie)](
-    cookies1: Cookies, 
-    cookies2: CookieJar | SimpleCookie, 
+def update_cookies[T: (SimpleCookie, CookieJar)](
+    cookies1: T, 
+    cookies2, 
     /, 
-) -> Cookies:
-    if isinstance(cookies1, CookieJar):
+) -> T:
+    if isinstance(cookies1, SimpleCookie):
         if isinstance(cookies2, SimpleCookie):
-            cookies: Iterable[Cookie | Morsel] = cookies2.values()
+            morsels: Iterable[tuple[str, Morsel]] = cookies2.items()
+        elif isinstance(cookies2, CookieJar):
+            morsels = ((cookie.name, to_morsel(cookie)) for cookie in cookies2)
+        elif isinstance(cookies2, Mapping):
+            morsels = ((k, v if isinstance(v, Morsel) else to_morsel(v)) for k, v in iter_items(cookies2))
+        else:
+            morsels = ((c.key, c) for c in (c if isinstance(c, Morsel) else to_morsel(c) for c in cookies2))
+        cookies1.update(morsels)
+    else:
+        if isinstance(cookies2, SimpleCookie):
+            cookies: Iterable = cookies2.values()
         elif isinstance(cookies2, CookieJar):
             cookies = cookies2
         elif isinstance(cookies2, Mapping):
-            cookies = ValuesView(cookies2)
+            cookies = (to_cookie(v, name=k) for k, v in iter_items(cookies2))
         else:
             cookies = cookies2
         set_cookie = cookies1.set_cookie
         for cookie in cookies:
-            if isinstance(cookie, Morsel):
-                cookie = morsel_to_cookie(cookie)
+            if not isinstance(cookie, Cookie):
+                cookie = to_cookie(cookie)
             set_cookie(cookie)
-    else:
-        if isinstance(cookies2, SimpleCookie):
-            morsels: Iterable[tuple[str, Morsel]] = cookies2.items()
-        elif isinstance(cookies2, CookieJar):
-            morsels = ((cookie.name, cookie_to_morsel(cookie)) for cookie in cookies2)
-        elif isinstance(cookies2, Mapping):
-            morsels = ((k, v if isinstance(v, Morsel) else cookie_to_morsel(v)) for k, v in ItemsView(cookies2))
-        else:
-            morsels = ((c.key, c) for c in (c if isinstance(c, Morsel) else cookie_to_morsel(c) for c in cookies2))
-        cookies1.update(morsels)
     return cookies1
 
 
-def cookies_str_to_dict(cookies: str, /) -> dict[str, str]:
-    return dict(cookie.split("=", 1) for cookie in CRE_COOKIE_SEP_split(cookies) if cookie)
-
-
-def cookies_dict_to_str(
-    cookies: Mapping[str, str] | Iterable[tuple[str, str]] | CookieJar | SimpleCookie, 
-    /, 
-) -> str:
-    if isinstance(cookies, CookieJar):
-        cookies = ((cookie.name, val) for cookie in cookies if (val := cookie.value) is not None)
-    elif isinstance(cookies, SimpleCookie):
-        cookies = ((name, morsel.value) for name, morsel in cookies.items())
-    else:
-        cookies = iter_items(cookies)
-    return "; ".join(f"{key}={val}" for key, val in cookies)
-
-
 def iter_resp_cookies(resp, /) -> Iterator[tuple[str, None | str]]:
     if hasattr(resp, "cookies"):
         cookies = resp.cookies
